#
#	TkForms.pm
#Mon Mar 24 14:49:47 CET 2003
#

package TkForms;
require 5.000;
@ISA = qw(MainWindow);

#
#	<> dependencies
#

use Tk;
use Tk::Text;
use	Tk::FileSelect;
use Tk::DialogBox;
use Tk::Checkbutton;
use TempFileNames;
use Set;

#
#	<> data structures
#

#
#	<> standard implementation
#

sub newWithConfig { my ($class, $config) = @_;
	my $self = bless($class->SUPER::new(), $class);
	# if no superclass is in place use the following instead
	# my $self = bless({}, $class);
	# own initialization follows

	my $c = $config;
	$self->{configDict} = {%{$c}};	# copy the config dict
	#$self->configure(-width => $c->{global}{windowSize}{w})
	#	if (defined($c->{global}{windowSize}{w}));

	foreach $field (@{$c->{fields}}) {
		my $method = 'addWidget'.$field->{type};
		Log("Adding widget via method $method", 3);
		&{$method}($self, $field);
		$self->{structure}{fields}{$field->{name}} = $field;
	}

	$self->addWidgetActions($c->{actions}, $c->{actionHandler});

	#use Data::Dumper;
	#print Dumper(collectValues($w, $self->{structure}));

	return $self;
}

#
#	<> method implementation
#

sub run { my ($self) = @_;
	MainLoop();
}

# field handlers are to be called from within the individual handlers
# through the $self->handleAction(...) method

sub handleAction { my ($self, $field) = @_;
	my $name = inverseMap($self->{structure}{widgets})->{$field};
	my $handler = $self->{configDict}{fieldHandler};

	Log("Field action $name". $handler, 3);
	if (defined($handler) && uc($self->{structure}{fields}{$name}{fieldAction}) eq 'YES') {
		$handler->($self->{structure}, $self, $name);
	}
}

# helping functions
sub fileSelectHandler { my ($w, $t) = @_;
	return sub { my ($fs, $f);
		my $p = splitPathDict(substr($t->get(), 0, -1)); # get('0.0', 'end') for Text
		Log("Path:".$p->{dir}, 1);
		$fs = $w->FileSelect(-directory => $p->{dir});
		$f = $fs->Show();
		$w->setWidgetFileSelect($t, $f) if ($f ne '');
		$w->handleAction($t);
	};
};
sub displayHelp { my ($w, $label, $helpText) = @_;
	my $d = $w->DialogBox(-title => 'Help --- '. $label, -buttons => ['OK']);
	my $t = $d->add('Text', -bg => 'white', height => 15, width => 60);
	$t->pack(-side => left, -fill => 'y', -expand => 1,);
	$t->insert("0.0", $helpText);
	$d->Show();
}

sub addFrameWithLabel { my ($self, $label) = @_;
 	my $f = $self->Frame()->pack(-side => 'top', -fill => 'both');
	$f->Label(-text => $label)->pack(-side => 'left');
	return $f;
}
sub addHelpToFrameFromDesc { my ($self, $f, $desc) = @_;
	$f->Button(-text => 'Help', -command => sub { displayHelp($self, $desc->{label},
		$desc->{help}) } )->pack(-side => 'right')
		if (defined($desc->{help}));
}

sub addWidgetFileSelect { my ($self, $desc) = @_;
 	my $f = $self->addFrameWithLabel($desc->{label});
	#my $t = $f->Text(height => 1, -bg => 'white');
	my $t = $f->Entry(-bg => 'white');

	my $fsb = $f->Button(-text => '...', -command => fileSelectHandler($self, $t));
	$fsb->pack(-side => 'left');
	$t->pack(side => 'left');
	$self->setWidgetFileSelect($t, $field->{default});
	$t->pack(-side => 'left', -fill => 'x', -expand => 1, -padx => 3);
#	$f->Balloon(-state => 'balloon')->attach($t, -ballonmsg => $desc->{help})
	$self->addHelpToFrameFromDesc($f, $desc);
	$self->{structure}{widgets}{$desc->{name}} = $t;
}
sub collectWidgetFileSelect { my ($self, $t) = @_;
	return $t->get(); #Text: substr($t->get('0.0', 'end'), 0, -1);
}
sub setWidgetFileSelect { my ($self, $t, $path) = @_;
	$t->delete('0.0', 'end');
	$t->insert('0.0', $path);
}

sub addWidgetPopup { my ($self, $desc) = @_;
	my $f = $self->addFrameWithLabel($desc->{label});
	my $p = $f->Optionmenu(-options => $desc->{options});
	$self->setWidgetPopup($p, $desc->{default}) if (defined($desc->{default}));
	$p->pack(side => 'left');

	$self->addHelpToFrameFromDesc($f, $desc);
	$self->{structure}{widgets}{$desc->{name}} = $p;
}
sub collectWidgetPopup { my ($self, $p, $d, $name) = @_;
	return ${$p->cget(-textvariable)};
}
sub setWidgetPopup { my ($self, $p, $f) = @_;
	${$p->cget(-textvariable)} = $f;
}
sub setWidgetPropertiesPopup { my ($self, $name, $p, $w) = @_;
	$w->configure(-options => $p);
}

sub addWidgetCheckbox { my ($self, $desc) = @_;
	my $f = $self->addFrameWithLabel($desc->{label});
	my $cb = $f->Checkbutton(-variable => \$self->{structure}{values}{$desc->{name}});
	$self->setWidgetCheckbox($cb, int($desc->{default})) if (defined($desc->{default}));
	$cb->pack(side => 'left');
	$self->addHelpToFrameFromDesc($f, $desc);
	$self->{structure}{widgets}{$desc->{name}} = $cb;
}
sub collectWidgetCheckbox { my ($self, $cb, $d, $name) = @_;
	return $self->{structure}{values}{$name};
}
sub setWidgetCheckbox { my ($self, $cb, $state) = @_;
    if ($state) { $cb->select(); } else { $cb->deselect(); }
}

sub addWidgetTextField { my ($self, $desc) = @_;
	my $f = $self->addFrameWithLabel($desc->{label});
	my $t = $f->Entry(-bg => 'white', -textvariable => \$self->{structure}{values}{$desc->{name}});
	$self->setWidgetTextField($t, $desc->{default}) if (defined($desc->{default}));
	$t->pack(side => 'left');
	$self->addHelpToFrameFromDesc($f, $desc);
	$self->{structure}{widgets}{$desc->{name}} = $t;
}
sub collectWidgetTextField { my ($self, $tf, $d, $name) = @_;
	return $self->{structure}{values}{$name};
}
sub setWidgetTextField { my ($self, $tf, $text) = @_;
	$tf->delete('0.0', 'end');
	$tf->insert('0.0', $text);
}


sub addWidgetActions { my ($self, $desc, $function) = @_;
 	my $f = $self->Frame()->pack(-side => 'top', -fill => 'both');

	$self->{structure}{mainWindow} = $self;
	foreach $a (@{$desc}) {
		my $b = $f->Button(-text => $a->{label});
		my $aName = $a->{name};
		$b->configure(-command => sub {
			$self->{structure}{action} = $aName;
			&{$function}($self->{structure}, $self, $aName);
		});
		$b->pack(-side => $a->{alignment});		
	}
}

sub valueForField { my ($self, $name) = @_;
	my $method = 'collectWidget'.$self->{structure}{fields}{$name}{type};
	return &{$method}($self, $self->{structure}{widgets}{$name}, $self->{structure}, $name);
}

sub collectValues { my ($self) = @_;
	my $values = {};
	foreach $w (keys %{$self->{structure}{widgets}}) {
		$values->{$w} = $self->valueForField($w);
	}
	return $values;
}

sub setPropertiesForField { my ($self, $name, $p) = @_;
	my $method = 'setWidgetProperties'.$self->{structure}{fields}{$name}{type};
	&{$method}($self, $name, $p, $self->{structure}{widgets}{$name});
}

sub setValues { my ($self, $dict) = @_;
	foreach $w (keys %{$self->{structure}{widgets}}) {
		my $method = 'setWidget'.$self->{structure}{fields}{$w}{type};
		Log("Setting Value $w: $dict->{$w}", 3);
		&{$method}($self, $self->{structure}{widgets}{$w}, $dict->{$w}, $self->{structure}, $w);
	}
}

1;
