#!/usr/local/bin/perl
#
#	mffinderTk.pl
#	Tue May 20 16:19:23 MEST 2003
#

use Getopt::Long;
use PropertyList;
use TempFileNames;
use TkForms;
use Set;

sub saveValues { my ($dict) = @_;
	my $stringRep = stringFromProperty($dict);
	my $savePath = firstDef($defaultsPath, map { -e $_? "$_/mffinderDefaults.cfg": undef() }
		("$ENV{HOME}/Library/Configs", '.'));
	Log("Saving config file to: $savePath");
	Log("Defaults:\n$stringRep", 3);
	writeFile($savePath, $stringRep);
}

@geneticModels = ( 'Additive', 'Codominant', 'Genotypes' );

sub executeMffinder { my ($v) = @_;
	my $tmpFile = tempFileName('/tmp/mffinderOutput', '.txt');
	my $cmd = "mffinder \"$v->{input}\" $v->{countOfLoci} $v->{minCount} $v->{minOR} $v->{maxOR} ".
		"> \"$v->{output}list.txt\"";
	my $nor = $v->{nor}? '--ree' : '';
	my $norApp = $v->{nor}? 'Nor' : '';
	my $gcmd = "perl mfVisualize.pl $nor --postscript --allLoci --maxOR $v->{orGraphics} "
		."-o \"$v->{output}Graphics$norApp\" "
		." < \"$v->{output}list.txt\"";
	Log($cmd);
	system($cmd);

	if ($v->{graphics}) {
		Log($gcmd);
		system($gcmd);

		my $op = splitPathDict($v->{output});
		# this is an ugly hack to differetiate UNIX/Windows <!><%>
		my $unix = -e '/usr';

		if ($unix) {	# conversion tools are unavailable under windows
			foreach $m (@geneticModels) {
				my $ccmd = "cd $op->{dir} ; ps2pdf -sPAPERSIZE=a4 "
					."\"$op->{file}Graphics$norApp-$m.ps\"";
				Log($ccmd);
				system($ccmd);
			}
		}

		if ($v->{show}) {
			my $viewer = firstDef($v->{viewer}, 'acroread');
			my $detach = $unix? ' &': '';

			my $vcmd = $viewer.' '. join(' ',
				map { "\"$v->{output}Graphics$norApp-$_.".($unix? 'pdf': 'ps').'"' } @geneticModels)
				. $detach;
			Log($vcmd);
			system($vcmd);
		}
	}
}

# action callback
sub doAction { my ($d, $mainWindow, $n) = @_;
	my $w = $d->{mainWindow};
	my $vs = $w->collectValues();
	SWITCH: {
		if ($d->{action} eq 'save') { saveValues($vs); last SWITCH; }
		if ($d->{action} eq 'cancel') { $w->destroy(); last SWITCH; }
		if ($d->{action} eq 'run') { executeMffinder($vs); last SWITCH; }
	}
}

#main $#ARGV @ARGV %ENV
	$result = GetOptions('simple' => \$defUndef, 'filter=s' => \$name,
		'int=i' => \$int, 'onOff!' => \$flag);
	$result = GetOptions('help' => \$help, 'h' => \$help);

	if ($help || !$result)
	{	printf("USAGE: %s\n", ($0 =~ m{/?([^/]*)$}o));
		exit(!$result);
	}

	my $plistFile = readFileFirstLocation("mffinderTk.cfg",
		(".", "$ENV{HOME}/Library/Configs", "/Library/Configs"));
	die "Config file CCtk.cfg not found" if (!defined($plistFile));
	$c = propertyFromString($plistFile);
	$c->{actionHandler} = \&doAction;
#	$c->{fieldHandler} = \&fieldAction;

	my $defaultsFile = readFileFirstLocation('mffinderDefaults.cfg',
		('.', "$ENV{HOME}/Library/Configs", '/Library/Configs'));
	my $defaults = defined($defaultsFile)? propertyFromString($defaultsFile): undef();
	Log($defaultsFile);

#	setFieldForName($c, 'options', [keys %{$ccconfig->{placements}}], 'placement');

	$f = TkForms->newWithConfig($c);
	$f->setValues($defaults) if (defined($defaults));
	$f->run();

exit(0);
